@@ -12,11 +12,6 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Ian F. Darwin and others.
- * 4. The name of the author may not be used to endorse or promote products
- *    derived from this software without specific prior written permission.
  *  
  * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
@@ -37,7 +32,7 @@
  * appear at fixed offsets into the file. Don't make HOWMANY
  * too high unless you have a very fast CPU.
  *
- * $Id: names.h,v 1.23 2003/10/14 19:29:55 christos Exp $
+ * $Id: names.h,v 1.25 2004/09/11 19:15:57 christos Exp $
  */
 
 /*
@@ -60,6 +55,7 @@
 #define	L_HTML	11		/* HTML */
 #define	L_BCPL	12		/* BCPL */
 #define	L_M4	13		/* M4 */
+#define	L_PO	14		/* PO */
 
 static const struct {
 	const char *human;
@@ -71,7 +67,7 @@ static const struct {
 	{ "make commands",				"text/x-makefile" },
 	{ "PL/1 program",				"text/x-pl1" },
 	{ "assembler program",				"text/x-asm" },
-	{ "English",					"text/plain, English" },
+	{ "English",					"text/plain" },
 	{ "Pascal program",				"text/x-pascal" },
 	{ "mail",					"text/x-mail" },
 	{ "news",					"text/x-news" },
@@ -79,6 +75,7 @@ static const struct {
 	{ "HTML document",				"text/html", },
 	{ "BCPL program",				"text/x-bcpl" },
 	{ "M4 macro language pre-processor",		"text/x-m4" },
+	{ "PO (gettext message catalogue)",             "text/x-po" },
 	{ "cannot happen error on names.h/types",	"error/x-error" },
 	{ 0, 0}
 };
@@ -125,6 +122,7 @@ static struct names {
 } names[] = {
 	/* These must be sorted by eye for optimal hit rate */
 	/* Add to this list only after substantial meditation */
+	{"msgid",	L_PO},
 	{"dnl",		L_M4},
 	{"import",	L_JAVA},
 	{"\"libhdr\"",	L_BCPL},
