@@ -37,7 +37,7 @@
 #include "readelf.h"
 
 #ifndef lint
-FILE_RCSID("@(#)$Id: readelf.c,v 1.45 2004/11/24 17:38:24 christos Exp $")
+FILE_RCSID("@(#)$Id: readelf.c,v 1.49 2005/07/14 02:22:25 christos Exp $")
 #endif
 
 #ifdef	ELFCORE
@@ -176,6 +176,8 @@ size_t	prpsoffsets32[] = {
 };
 
 size_t	prpsoffsets64[] = {
+	16,		/* FreeBSD, 64-bit */
+	40,             /* Linux (tested on core from 2.4.x) */
        120,		/* SunOS 5.x, 64-bit */
 };
 
@@ -487,6 +489,19 @@ donote(struct magic_set *ms, unsigned char *nbuf, size_t offset, size_t size,
 		return size;
 	}
 
+	if (namesz == 10 && strcmp((char *)&nbuf[noff], "DragonFly") == 0 &&
+	    nh_type == NT_DRAGONFLY_VERSION && descsz == 4) {
+		uint32_t desc;
+		if (file_printf(ms, ", for DragonFly") == -1)
+			return size;
+		(void)memcpy(&desc, &nbuf[doff], sizeof(desc));
+		desc = getu32(swap, desc);
+		if (file_printf(ms, " %d.%d.%d", desc / 100000,
+		    desc / 10000 % 10, desc % 10000) == -1)
+			return size;
+		return size;
+	}
+
 	/*
 	 * Sigh.  The 2.0.36 kernel in Debian 2.1, at
 	 * least, doesn't correctly implement name
